// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "NewTrueType.hpp"

using namespace Dawn;

NewTrueTypeFace::NewTrueTypeFace() {
}

usagelockid_t NewTrueTypeFace::lock(struct NewTrueTypeFaceTextureStyle style) {
  // Find by style
  NewTrueTypeFaceTexture *texture = nullptr;
  auto existing = this->texturesByStyle.find(style);
  if(existing == this->texturesByStyle.end()) {
    // Load face
    FT_Face face;
    assertUnreachable();

    // Does not exist, create it.
    texture = new NewTrueTypeFaceTexture(this, face, style);
    this->textures.push_back(texture);
    this->texturesByStyle[style] = texture;
    
    // Cleanup face
    FT_Done_Face(face);
  } else {
    texture = existing->second;
  }

  assertNotNull(texture);
  auto lock = texture->locks.createLock();
  this->locksByStyle[style].push_back(lock);
  this->stylesByLock[lock] = style;

  return lock;
}

NewTrueTypeFaceTexture * NewTrueTypeFace::getTexture(usagelockid_t lock) {
  auto styleByLock = this->stylesByLock.find(lock);
  assertTrue(styleByLock != this->stylesByLock.end());
  auto texture = this->texturesByStyle.find(styleByLock->second);
  assertTrue(texture != this->texturesByStyle.end());
  return texture->second; 
}

void NewTrueTypeFace::unlock(usagelockid_t lock) {
  auto texture = this->getTexture(lock);
  texture->locks.removeLock(lock);
  this->stylesByLock.erase(lock);
  auto it = std::find(this->locksByStyle[texture->style].begin(), this->locksByStyle[texture->style].end(), lock);
  assertTrue(it != this->locksByStyle[texture->style].end());
  this->locksByStyle[texture->style].erase(it);
}

NewTrueTypeFace::~NewTrueTypeFace() {
  auto it = this->textures.begin();
  while(it != this->textures.end()) {
    delete *it;
    it = this->textures.erase(it);
  }
}