// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "NewTrueTypeFaceTexture.hpp"

namespace Dawn {
  class NewTrueTypeFace {
    protected:
      std::vector<NewTrueTypeFaceTexture*> textures;
      std::map<struct NewTrueTypeFaceTextureStyle, NewTrueTypeFaceTexture*> texturesByStyle;
      std::map<struct NewTrueTypeFaceTextureStyle, std::vector<usagelockid_t>> locksByStyle;
      std::map<usagelockid_t, struct NewTrueTypeFaceTextureStyle> stylesByLock;

    public:
      /**
       * Create a new TrueTypeFace object. TrueType face holds multiple textures
       * of various styles of this face. This face may be "Arial Bold" and have
       * a texture for "Arial Bold 12", "Arial Bold 14", etc.
       */
      NewTrueTypeFace();

      /**
       * Locks a texture for use. This will create a new texture if one does not
       * exist for the given style.
       * 
       * @param style Style to lock.
       * @return A unique lock ID for this style.
       */
      usagelockid_t lock(struct NewTrueTypeFaceTextureStyle style);

      /**
       * Unlocks a previously created lock. If the texture becomes unused it 
       * will be deleted.
       * 
       * @param lock Lock to remove.
       */
      void unlock(usagelockid_t lock);

      /**
       * Gets a texture from a lock.
       * 
       * @param lock Lock to get texture from.
       * @return Texture from lock.
       */
      NewTrueTypeFaceTexture * getTexture(usagelockid_t lock);

      /**
       * Destroys this TrueTypeFace object, and all textures associated with it.
       */
      ~NewTrueTypeFace();

      friend class NewTrueType;
  };
}