// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "display/font/truetype/NewTrueTypeShared.hpp"
#include "util/mathutils.hpp"
#include "display/Texture.hpp"
#include "util/UsageLock.hpp"

namespace Dawn {
  class NewTrueTypeFace;
  class NewTrueType;

  typedef uint32_t newtruetypelock_t;

  struct NewTrueTypeCharacter {
    int32_t advanceX;
    int32_t advanceY;
    glm::vec2 bitmapSize;
    glm::vec2 bitmapPosition;
    float_t textureY;
  };

  struct NewTrueTypeFaceTextureStyle {
    uint32_t fontSize;
    flag_t style;
  };

  class NewTrueTypeFaceTexture {
    protected:
      FT_Face *face;
      NewTrueTypeFace *trueTypeFace;
      std::map<FT_ULong, struct NewTrueTypeCharacter> characterData;
      struct NewTrueTypeFaceTextureStyle style;
      UsageLock locks;

    public:
      Texture texture;

      /**
       * Construct a new New True Type Face Texture object
       * 
       * @param trueTypeFace Face that this texture belongs to.
       * @param face The freetype face object.
       * @param style Style that this font has, used for locking.
       */
      NewTrueTypeFaceTexture(
        NewTrueTypeFace *trueTypeFace,
        FT_Face &face,
        struct NewTrueTypeFaceTextureStyle style
      );

      /**
       * Returns the character data for the given character.
       * 
       * @param c Character to get data for.
       * @return The Character data for the given character.
       */
      struct NewTrueTypeCharacter getCharacterData(FT_ULong c);

      /**
       * Destroys this true type face texture.
       */
      ~NewTrueTypeFaceTexture();

      friend class NewTrueTypeFace;
      friend class NewTrueType;
  };
}