// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/components/ui/UIComponentRenderable.hpp"
#include "display/mesh/QuadMesh.hpp"

namespace Dawn {
  struct UILabelStyle {
    struct Color;
    UILabelFontDef *fontDef;
  };

  class UILabelNew : public UIComponentRenderable {
    private:
      Mesh mesh;
      FontShaderBuffer shaderBuffer;

      FT_Face face;
      FT_Face faceBold;
      FT_Face faceItalics;

      struct UILabelFontDef defFace;
      struct UILabelFontDef defFaceBold;
      struct UILabelFontDef defFaceItalics;

      /**
       * Buffers the quads for the given text and updates the progressing values
       * as the buffer process continues.
       * 
       * @param text Text to buffer.
       * @param bufferData The output quad mappings for the text.
       * @param fontDef The font definition to use.
       * @param position The 2D position to buffer the quads at.
       * @param quadStart The starting quad index.
       * @param partIndex The part index to store for each quad buffered.
       * @return The number of quads buffered, not the string length.
       */
      int32_t bufferQuads(
        std::string text,
        struct FontShaderBufferData &bufferData,
        struct UILabelFontDef &fontDef,
        glm::vec2 &position,
        int32_t quadStart,
        int32_t partIndex
      );

    public:
      UILabelNew(SceneItem *item);

      void onStart() override;
      std::vector<struct ShaderPassItem> getUIRenderPasses() override;
      float_t getWidth() override;
      float_t getHeight() override;
      float_t getContentWidth() override;
      float_t getContentHeight() override;
  };
}